/************************************************************************
 *
 *  ListStyleConverter.java
 *
 *  Copyright: 2002-2015 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 1.6 (2015-06-03)
 *
 */

package w2phtml.xhtml.style;

import java.util.Enumeration;
//import java.util.Hashtable;

import w2phtml.office.ListStyle;
import w2phtml.office.OfficeReader;
import w2phtml.office.OfficeStyle;
import w2phtml.office.OfficeStyleFamily;
import w2phtml.office.StyleWithProperties;
import w2phtml.office.XMLString;
import w2phtml.util.CSVList;
import w2phtml.xhtml.Converter;
import w2phtml.xhtml.StyleInfo;
import w2phtml.xhtml.XhtmlConfig;
import w2phtml.xhtml.XhtmlStyleMapItem;
import w2phtml.xhtml.style.properties.Properties;

/**
 * This class converts OpenDocument list styles to
 * CSS2 styles (currently, actually CSS1).
 */
public class ListStyleParser extends StyleParser {

		private boolean inlineCSS;
    /** Create a new <code>ListStyleConverter</code>
     *  @param ofr an <code>OfficeReader</code> to read style information from
     *  @param config the configuration to use
     *  @param converter the main <code>Converter</code> class
     *  @param nType the type of xhtml to use
     */
    public ListStyleParser(OfficeReader ofr, XhtmlConfig config, Converter converter, int nType) {
        super(ofr,config,converter,nType);
        this.styleMap = config.getXListStyleMap();
        this.inlineCSS = config.inlineCSS();
    }

    public void applyStyle(int nLevel, String sStyleName, StyleInfo info) {
        ListStyle style = ofr.getListStyle(sStyleName);
        if (style!=null) {
            if (style.isAutomatic()) {
                applyStyle(nLevel,style.getParentName(),info);
                cssList(style,nLevel,info.props);
            }
            else {
                String sDisplayName = style.getDisplayName();
                if (styleMap.contains(sDisplayName)) {
                	XhtmlStyleMapItem map = styleMap.get(sDisplayName);
                	if (map.sElement.length()>0) {
                		info.sTagName = map.sElement;
                	}
                    if (!"(none)".equals(map.sCss)) {
                        info.sClass = map.sCss;
                    }
                }
                else {
        					styleNames.addName(sDisplayName);
        					info.sClass = "listlevel"+Integer.toString(nLevel)+styleNames.getName(sDisplayName);
                }
            }
        }
    }
	
	

    /** <p>Convert style information for used styles</p>
     *  @param sIndent a String of spaces to add before each line
     */
  	public String composeStyleDeclarations() {
  		if (inlineCSS) {
  			return "";
  		}
  		StringBuilder buf = new StringBuilder();
  		Enumeration<String> names = styleNames.keys();
  		while (names.hasMoreElements()) {
  			String displayName = names.nextElement();
  			buf.append(composeDeclaration(displayName));
				if (config.listFormatting() == XhtmlConfig.HARD_LABELS) {
					buf.append(composeHardDeclaration(displayName));
				}
  		}
  		return buf.toString();
  	}
  	public String composeDeclaration(String displayName) {
  		StringBuilder buf = new StringBuilder();
  		ListStyle style = (ListStyle) getStyles().getStyleByDisplayName(displayName);
  		if (!style.isAutomatic()) {
  			for (int level = 1; level < 10; level++) {
  				String selector = composeSelector(displayName, level);
  				String declarationBlock = composeDeclarationBlock(displayName, level);
  				if (!selector.isEmpty() && !declarationBlock.isEmpty()) {
  					buf.append(createDeclaration(selector, declarationBlock));	
  				}
  			}
  		}
  		return buf.toString();
  	}
  	
  	public String composeHardDeclaration(String displayName) {
  		StringBuilder buf = new StringBuilder();
  		ListStyle style = (ListStyle) getStyles().getStyleByDisplayName(displayName);
  		if (!style.isAutomatic()) {
  			for (int level = 1; level < 10; level++) {
  				String declarationBlock = composeHardDeclarationBlock(displayName, level);
					String selector = composeHardSelector(displayName, level);
  					// Apply left margin and text indent to the paragraphs contained in
  					// the list
  				if (!declarationBlock.isEmpty() && !selector.isEmpty()) {
  					buf.append(createDeclaration(selector, declarationBlock));
  				}
  			}
  		}
  		return buf.toString();
  	}

		private String  composeHardSelector(String displayName, int level) {
			styleNames.addName(displayName);
			return ".listlevel" + level + styleNames.getName(displayName) + " p ";
		}
		
		private String composeSelector(String displayName, int level) {
			styleNames.addName(displayName);
			return ".listlevel" + level + styleNames.getName(displayName);
		}

		private String composeHardDeclarationBlock(String displayName, int level) {
  		ListStyle style = (ListStyle) getStyles().getStyleByDisplayName(displayName);
			CSVList props = new CSVList(";");
			cssListParMargins(style, level, props);
			return props.toString();
		}

		private String composeDeclarationBlock(String displayName, int level) {
  		ListStyle style = (ListStyle) getStyles().getStyleByDisplayName(displayName);
  		Properties props = new Properties(";");
			cssList(style, level, props);
			return props.toString();
		}

		public void readParentStyle(int level,String styleName, StyleInfo info) {
			if (!inlineCSS || styleName == null) {
				return;
			}
			ListStyle style = (ListStyle) getStyles().getStyle(styleName);
			cssList(style, level, info.props);
		}
	
    /** Get the family of list styles
     *  @return the style family
     */
    public OfficeStyleFamily getStyles() {
        return ofr.getListStyles();
    }
    
    private void cssList(ListStyle style, int nLevel, Properties props){
        // translates "list" style properties for a particular level
        // Mozilla does not seem to support the "marker" mechanism of CSS2
        // so we will stick with the simpler CSS1-like list style properties
        props.addProperty("margin-top","0");
        props.addProperty("margin-bottom","0");
        if (config.listFormatting()!=XhtmlConfig.HARD_LABELS) {
        	// Export the numbering to CSS1
        	String sLevelType = style.getLevelType(nLevel);
        	if (XMLString.TEXT_LIST_LEVEL_STYLE_NUMBER.equals(sLevelType)) {
        		// Numbering style, get number format
        		String sNumFormat = style.getLevelProperty(nLevel,XMLString.STYLE_NUM_FORMAT);
        		if ("1".equals(sNumFormat)) { props.addProperty("list-style-type","decimal"); }
        		else if ("i".equals(sNumFormat)) { props.addProperty("list-style-type","lower-roman"); }
        		else if ("I".equals(sNumFormat)) { props.addProperty("list-style-type","upper-roman"); }
        		else if ("a".equals(sNumFormat)) { props.addProperty("list-style-type","lower-alpha"); }
        		else if ("A".equals(sNumFormat)) { props.addProperty("list-style-type","upper-alpha"); }

        		
        	}
        	else if (XMLString.TEXT_LIST_LEVEL_STYLE_BULLET.equals(sLevelType)) {
        		// Bullet. We can only choose from disc, bullet and square
        		switch (nLevel % 3) {
        		case 1: props.addProperty("list-style-type","disc"); break;
        		case 2: props.addProperty("list-style-type","circle"); break;
        		case 0: props.addProperty("list-style-type","square"); break;
        		}
        	}
        	else if (XMLString.TEXT_LIST_LEVEL_STYLE_IMAGE.equals(sLevelType)) {
        		// Image. TODO: Handle embedded images
        		String sHref = style.getLevelProperty(nLevel,XMLString.XLINK_HREF);
        		if (sHref!=null) { props.addProperty("list-style-image","url('"+sHref+"')"); }
        	}
        }
        else {
        	// No numbering generated by the list; we add hard numbering to the paragraph
        	props.addProperty("list-style-type","none");

        	// In this case we also set the left margin for the list
        	// For real styles the margins are applied to the paragraphs
        	// This is more tricky for hard styles, so we use a default left margin on the list
        	if (style.isAutomatic() && nLevel>1) {
        		props.addProperty("margin-left", "2em");
        	}
        	else {
        		props.addProperty("margin-left","0");
        	}

        	// Also reset the padding (some browsers use a non-zero default value)
        	props.addProperty("padding-left", "0");
        }
        
        // We don't want floats to pass a list to the left (Mozilla and IE both
        //handles this terribly!)
        props.addProperty("clear","left");
    		String textStyleName = style.getLevelProperty(nLevel, XMLString.TEXT_STYLE_NAME);
    		if (textStyleName != null) {
    			StyleWithProperties charStyle = (StyleWithProperties) ofr.getTextStyles().getStyle(textStyleName);
    			getTextSP().cssText(charStyle, props, true);
    		}
    }
    
    private void cssListParMargins(ListStyle style, int nLevel, CSVList props){
		// Instead margin is applied to the paragraphs in the list, more precisely the list style defines a
    	// left margin and a text indent to *replace* the values from the paragraph style
		String sMarginLeft = style.getLevelStyleProperty(nLevel, XMLString.FO_MARGIN_LEFT);
		if (sMarginLeft!=null) { 
			props.addValue("margin-left", sMarginLeft);
		}
		else {
			props.addValue("margin-left", "0");
		}
		String sTextIndent = style.getLevelStyleProperty(nLevel, XMLString.FO_TEXT_INDENT);
		if (sTextIndent!=null) { 
			props.addValue("text-indent", sTextIndent);
		}
		else {
			props.addValue("text-indent", "0");
		}
    }


	
}
